@@ -14,6 +14,7 @@ import warp_pb2_grpc
 import interceptors
 import prefs
 import util
+import misc
 import transfers
 import auth
 from ops import SendOp, ReceiveOp
@@ -242,12 +243,12 @@ class RemoteMachine(GObject.Object):
                     self.rpc_call(self.update_remote_machine_avatar)
 
                     # Online loop
+                    logging.info("Connected to %s" % self.display_hostname)
                     while not self.channel_keepalive.is_set():
                         self.channel_keepalive.wait(.5)
                     ##
 
                 except Exception as e:
-                    print("exception")
                     self.set_remote_status(RemoteStatus.UNREACHABLE)
 
                     if isinstance(e, grpc.FutureTimeoutError):
@@ -368,19 +369,27 @@ class RemoteMachine(GObject.Object):
             self.emit_machine_info_changed()
             self.set_remote_status(RemoteStatus.ONLINE)
         
-        future = self.stub.GetRemoteMachineInfo.future(warp_pb2.LookupName(id=self.local_ident,
-                                                                           readable_name=util.get_hostname()))
+        future = self.stub.GetRemoteMachineInfo.future(
+            warp_pb2.LookupName(
+                id=self.local_ident,
+                readable_name=util.get_hostname()
+            )
+        )
         future.add_done_callback(get_info_finished)
 
     # Run in thread pool
     def update_remote_machine_avatar(self):
         logging.debug("Remote RPC: calling GetRemoteMachineAvatar on '%s'" % self.display_hostname)
-        iterator = self.stub.GetRemoteMachineAvatar(warp_pb2.LookupName(id=self.local_ident,
-                                                                        readable_name=util.get_hostname()))
+        iterator = self.stub.GetRemoteMachineAvatar(
+            warp_pb2.LookupName(
+                id=self.local_ident,
+                readable_name=util.get_hostname()
+            )
+        )
         loader = None
         try:
             for info in iterator:
-                if loader == None:
+                if loader is None:
                     loader = util.CairoSurfaceLoader()
                 loader.add_bytes(info.avatar_chunk)
         except grpc.RpcError as e:
@@ -388,7 +397,7 @@ class RemoteMachine(GObject.Object):
 
         self.get_avatar_surface(loader)
 
-    @util._idle
+    @misc._idle
     def get_avatar_surface(self, loader=None):
         # This needs to be on the main loop, or else we get an x error
         if loader:
@@ -405,17 +414,22 @@ class RemoteMachine(GObject.Object):
 
         logging.debug("Remote RPC: calling TransferOpRequest on '%s'" % (self.display_hostname))
 
-        transfer_op = warp_pb2.TransferOpRequest(info=warp_pb2.OpInfo(ident=op.sender,
-                                                                      timestamp=op.start_time,
-                                                                      readable_name=util.get_hostname(),
-                                                                      use_compression=prefs.use_compression()),
-                                                 sender_name=op.sender_name,
-                                                 receiver=self.ident,
-                                                 size=op.total_size,
-                                                 count=op.total_count,
-                                                 name_if_single=op.description,
-                                                 mime_if_single=op.mime_if_single,
-                                                 top_dir_basenames=op.top_dir_basenames)
+        transfer_op = warp_pb2.TransferOpRequest(
+            info=warp_pb2.OpInfo(
+                ident=op.sender,
+                timestamp=op.start_time,
+                readable_name=util.get_hostname(),
+                use_compression=prefs.use_compression(),
+            ),
+            sender_name=op.sender_name,
+            receiver=self.ident,
+            size=op.total_size,
+            count=op.total_count,
+            name_if_single=op.description,
+            mime_if_single=op.mime_if_single,
+            top_dir_basenames=op.top_dir_basenames
+        )
+
         self.stub.ProcessTransferOpRequest(transfer_op)
 
     # Run in thread pool
@@ -426,9 +440,13 @@ class RemoteMachine(GObject.Object):
             name = op.sender
         else:
             name = self.local_ident
-        self.stub.CancelTransferOpRequest(warp_pb2.OpInfo(timestamp=op.start_time,
-                                                          ident=name,
-                                                          readable_name=util.get_hostname()))
+        self.stub.CancelTransferOpRequest(
+            warp_pb2.OpInfo(
+                timestamp=op.start_time,
+                ident=name,
+                readable_name=util.get_hostname()
+            )
+        )
         op.set_status(OpStatus.CANCELLED_PERMISSION_BY_SENDER if by_sender else OpStatus.CANCELLED_PERMISSION_BY_RECEIVER)
 
     # Run in thread pool
@@ -457,7 +475,11 @@ class RemoteMachine(GObject.Object):
         def report_receive_error(error):
             op.file_iterator = None
 
-            if error == None:
+            # Get rid of any toplevel file/folder if the transfer stops prematurely,
+            # so it or its children 
+            receiver.clean_current_top_dir_file()
+
+            if error is None:
                 return
 
             op.set_error(error)
@@ -466,7 +488,7 @@ class RemoteMachine(GObject.Object):
                 # If we leave an io stream open, it locks the location.  For instance,
                 # if this was a mounted location, we wouldn't be able to terminate until
                 # we closed warp.
-                if receiver.current_stream != None:
+                if receiver.current_stream is not None:
                     receiver.current_stream.close()
             except GLib.Error:
                 pass
@@ -476,6 +498,8 @@ class RemoteMachine(GObject.Object):
             op.stop_transfer()
 
         try:
+            receiver.clean_existing_files()
+
             for data in op.file_iterator:
                 receiver.receive_data(data)
 
@@ -486,31 +510,25 @@ class RemoteMachine(GObject.Object):
                           (op.total_count, GLib.format_size(op.total_size),\
                            util.precise_format_time_span(GLib.get_monotonic_time() - start_time)))
 
-            if receiver.remaining_files > 0:
-                raise ReceiveError(_("Transfer completed, but the number of files received is less than the original request size (expected %d, received %d)"
-                                       % (op.total_count, op.total_count - receiver.remaining_files)),
+            if op.remaining_count > 0:
+                raise ReceiveError("Transfer completed, but the number of files received is less than the original request size (expected %d, received %d)"
+                                       % (op.total_count, op.total_count - receiver.remaining_count),
                                    fatal=False)
+            op.set_status(OpStatus.FINISHED)
         except grpc.RpcError as e:
             if e.code() == grpc.StatusCode.CANCELLED:
                 report_receive_error(None)
-                return
             else:
                 report_receive_error(e)
-                return
         except ReceiveError as e:
             if e.fatal:
                 report_receive_error(e)
-                return
             else:
                 logging.critical(str(e))
                 op.set_error(e)
                 op.set_status(OpStatus.FINISHED_WARNING)
-                return
         except Exception as e:
             report_receive_error(e)
-            return
-
-        op.set_status(OpStatus.FINISHED)
 
     # Run in thread pool
     def stop_transfer_op(self, op, by_sender=False, lost_connection=False):
@@ -544,25 +562,30 @@ class RemoteMachine(GObject.Object):
         if not lost_connection:
             # We don't need to send this if it's a connection loss, the other end will handle
             # its own cleanup.
-            opinfo = warp_pb2.OpInfo(timestamp=op.start_time,
-                                     ident=name,
-                                     readable_name=util.get_hostname())
+            opinfo = warp_pb2.OpInfo(
+                timestamp=op.start_time,
+                ident=name,
+                readable_name=util.get_hostname()
+            )
             self.stub.StopTransfer(warp_pb2.StopInfo(info=opinfo, error=op.error_msg != ""))
 
     # Op handling (run in thread pool)
-    def send_files(self, uri_list):
+    def send_files(self, uri_list, dbus_sent=False):
         def _send_files(uri_list):
-            op = SendOp(self.local_ident,
-                        self.ident,
-                        self.display_name,
-                        uri_list)
+            op = SendOp(
+                self.local_ident,
+                self.ident,
+                self.display_name,
+                uri_list
+            )
+            op.dbus_op = dbus_sent
             self.add_op(op)
             op.prepare_send_info()
 
         util.add_to_recents_if_single_selection(uri_list)
         self.rpc_call(_send_files, uri_list)
 
-    @util._idle
+    @misc._idle
     def add_op(self, op):
         if op not in self.transfer_ops:
             self.transfer_ops.append(op)
@@ -581,15 +604,22 @@ class RemoteMachine(GObject.Object):
         op.connect("active", lambda op: set_busy())
 
         self.emit_ops_changed()
+
+        # For now, only bad base filenames cause this (failed util.test_resolved_path_safety())
+        # We let it get this far so the UI has something to show the user.
+        if op.status == OpStatus.FAILED_UNRECOVERABLE:
+            op.decline_transfer_request()
+            return
+
         self.check_for_autostart(op)
 
-    @util._idle
+    @misc._idle
     def notify_remote_machine_of_new_op(self, op):
         if op.status == OpStatus.WAITING_PERMISSION:
             if op.direction == TransferDirection.TO_REMOTE_MACHINE:
                 self.rpc_call(self.send_transfer_op_request, op)
 
-    @util._idle
+    @misc._idle
     def check_for_autostart(self, op):
         if op.status == OpStatus.WAITING_PERMISSION:
             if isinstance(op, ReceiveOp) and \
@@ -602,7 +632,7 @@ class RemoteMachine(GObject.Object):
         self.transfer_ops.remove(op)
         self.emit_ops_changed()
 
-    @util._idle
+    @misc._idle
     def emit_ops_changed(self, op=None):
         self.emit("ops-changed")
 
@@ -617,7 +647,7 @@ class RemoteMachine(GObject.Object):
                     op.error_msg = _("Connection has been lost")
                     op.set_status(OpStatus.FAILED_UNRECOVERABLE)
 
-    @util._idle
+    @misc._idle
     def op_command_issued(self, op, command):
         # send
         if command == OpCommand.CANCEL_PERMISSION_BY_SENDER:
@@ -639,7 +669,7 @@ class RemoteMachine(GObject.Object):
         elif command == OpCommand.STOP_TRANSFER_BY_RECEIVER:
             self.rpc_call(self.stop_transfer_op, op, by_sender=False)
 
-    @util._idle
+    @misc._idle
     def op_focus(self, op):
         self.emit("focus-remote")
 
