@@ -22,6 +22,7 @@ import remote
 import remote_registration
 import prefs
 import util
+import misc
 import transfers
 from ops import ReceiveOp
 from util import TransferDirection, OpStatus, RemoteStatus
@@ -94,7 +95,7 @@ class Server(threading.Thread, warp_pb2_grpc.WarpServicer, GObject.Object):
 
         self.zeroconf = Zeroconf(interfaces=[self.ip_info.ip4_address])
 
-        self.service_ident = auth.get_singleton().get_ident()
+        self.service_ident = prefs.get_connect_id()
         self.service_name = "%s.%s" % (self.service_ident, SERVICE_TYPE)
 
         # If this process is killed (either kill or network issue), the service
@@ -140,7 +141,6 @@ class Server(threading.Thread, warp_pb2_grpc.WarpServicer, GObject.Object):
             return
 
         ident = name.partition(".%s" % SERVICE_TYPE)[0]
-        self.remote_registrar.cancel_registration(ident)
 
         try:
             remote = self.remote_machines[ident]
@@ -199,7 +199,7 @@ class Server(threading.Thread, warp_pb2_grpc.WarpServicer, GObject.Object):
             try:
                 machine = self.remote_machines[ident]
                 machine.has_zc_presence = True
-                logging.debug(">>> Discovery: existing remote: %s (%s:%d)"
+                logging.info(">>> Discovery: existing remote: %s (%s:%d)"
                                   % (machine.display_hostname, remote_ip_info.ip4_address, info.port))
 
                 # If the remote truly is the same one (our service info just dropped out
@@ -208,6 +208,8 @@ class Server(threading.Thread, warp_pb2_grpc.WarpServicer, GObject.Object):
                 # which we'll need when our supposedly existing connection tries to continue
                 # pinging. It will fail out and restart the connection loop, and will need
                 # this updated one.
+
+                # This blocks the zeroconf thread.
                 if not self.remote_registrar.register(ident, remote_hostname, remote_ip_info, info.port, auth_port, api_version) or self.server_thread_keepalive.is_set():
                     logging.warning("Register failed, or the server was shutting down during registration, ignoring remote %s (%s:%d) auth port: %d"
                                      % (remote_hostname, remote_ip_info.ip4_address, info.port, auth_port))
@@ -243,7 +245,7 @@ class Server(threading.Thread, warp_pb2_grpc.WarpServicer, GObject.Object):
                     if not found:
                         break
 
-                logging.debug(">>> Discovery: new remote: %s (%s:%d)"
+                logging.info(">>> Discovery: new remote: %s (%s:%d)"
                                   % (display_hostname, remote_ip_info.ip4_address, info.port))
 
                 machine = remote.RemoteMachine(ident,
@@ -254,8 +256,9 @@ class Server(threading.Thread, warp_pb2_grpc.WarpServicer, GObject.Object):
                                                self.service_ident,
                                                api_version)
 
+                # This blocks the zeroconf thread. Registration will timeout
                 if not self.remote_registrar.register(ident, remote_hostname, remote_ip_info, info.port, auth_port, api_version) or self.server_thread_keepalive.is_set():
-                    logging.warning("Register failed, or the server was shutting down during registration, ignoring remote %s (%s:%d) auth port: %d"
+                    logging.debug("Register failed, or the server was shutting down during registration, ignoring remote %s (%s:%d) auth port: %d"
                                      % (remote_hostname, remote_ip_info.ip4_address, info.port, auth_port))
                     return
 
@@ -277,7 +280,7 @@ class Server(threading.Thread, warp_pb2_grpc.WarpServicer, GObject.Object):
     def run(self):
         logging.debug("Server: starting server on %s (%s)" % (self.ip_info.ip4_address, self.ip_info.iface))
         logging.info("Using api version %s" % config.RPC_API_VERSION)
-        logging.info("Our uuid: %s" % auth.get_singleton().get_ident())
+        logging.info("Our uuid: %s" % prefs.get_connect_id())
 
 
         self.remote_registrar = remote_registration.Registrar(self.ip_info, self.port, self.auth_port)
@@ -367,14 +370,36 @@ class Server(threading.Thread, warp_pb2_grpc.WarpServicer, GObject.Object):
     def add_receive_op_to_remote_machine(self, op):
         self.remote_machines[op.sender].add_op(op)
 
-    @util._idle
+    @misc._idle
     def remote_ops_changed(self, remote_machine):
         self.emit("remote-machine-ops-changed", remote_machine.ident)
 
     def list_remote_machines(self):
         return self.remote_machines.values()
 
-    @util._idle
+    def get_active_op_count(self, incoming_only=False):
+        count = 0
+
+        for machine in self.remote_machines.values():
+            if machine.status != RemoteStatus.ONLINE:
+                continue
+            for op in machine.transfer_ops:
+                if incoming_only and not isinstance(op, ReceiveOp):
+                    continue
+                if op.status == OpStatus.TRANSFERRING:
+                    count += 1
+
+        return count
+
+    def cancel_all_ops(self):
+        for machine in self.remote_machines.values():
+            if machine.status != RemoteStatus.ONLINE:
+                continue
+            for op in machine.transfer_ops:
+                if op.status == OpStatus.TRANSFERRING:
+                    op.stop_transfer()
+
+    @misc._idle
     def idle_emit(self, signal, *callback_data):
         self.emit(signal, *callback_data)
 
@@ -451,7 +476,7 @@ class Server(threading.Thread, warp_pb2_grpc.WarpServicer, GObject.Object):
         try:
             remote_machine = self.remote_machines[request.info.ident]
         except KeyError as e:
-            logging.warning("Received transfer op request for unknown op: %s" % e)
+            logging.warning("Received transfer op request for unknown remote: %s" % e)
             return
 
         for existing_op in remote_machine.transfer_ops:
@@ -474,7 +499,7 @@ class Server(threading.Thread, warp_pb2_grpc.WarpServicer, GObject.Object):
         op.receiver_name = request.receiver_name
         op.status = OpStatus.WAITING_PERMISSION
         op.total_size = request.size
-        op.total_count = request.count
+        op.total_count = op.remaining_count = request.count
         op.mime_if_single = request.mime_if_single
         op.name_if_single = request.name_if_single
         op.top_dir_basenames = request.top_dir_basenames
@@ -537,7 +562,7 @@ class Server(threading.Thread, warp_pb2_grpc.WarpServicer, GObject.Object):
         sender = transfers.FileSender(op, request.timestamp, cancellable)
 
         def transfer_done():
-            if sender.error != None:
+            if sender.error is not None:
                 op.set_error(sender.error)
                 op.set_status(OpStatus.FAILED_UNRECOVERABLE)
             elif op.file_send_cancellable.is_set():
@@ -566,7 +591,7 @@ class Server(threading.Thread, warp_pb2_grpc.WarpServicer, GObject.Object):
             op.error_msg = _("An error occurred on the remote machine")
 
         if op.direction == TransferDirection.TO_REMOTE_MACHINE:
-            if op.file_send_cancellable != None:
+            if op.file_send_cancellable is not None:
                 op.file_send_cancellable.set()
             logging.debug("Server: sender received stop transfer by receiver: %s" % op.error_msg)
             if op.error_msg == "":
