@@ -12,6 +12,7 @@ from gi.repository import GLib, Gio, GObject
 import util
 from util import FileType, ReceiveError
 import prefs
+import misc
 import warp_pb2
 
 _ = gettext.gettext
@@ -168,7 +169,6 @@ class FileReceiver(GObject.Object):
     def __init__(self, op):
         super(FileReceiver, self).__init__()
         self.save_path = prefs.get_save_path()
-        self.save_path_obj = Path(self.save_path).resolve()
         self.op = op
         self.preserve_perms = prefs.preserve_permissions() and util.save_folder_is_native_fs()
         self.preserve_timestamp = prefs.preserve_timestamp() and util.save_folder_is_native_fs()
@@ -181,30 +181,47 @@ class FileReceiver(GObject.Object):
         self.current_mtime = 0
         self.current_mtime_usec = 0
 
-        self.remaining_files = op.total_count
-        self.remaining_bytes = op.total_size
-
-        for name in op.top_dir_basenames:
-            try:
-                path = os.path.join(self.save_path, name)
-                if os.path.isdir(path): # file not found is ok
-                    shutil.rmtree(path)
-                else:
-                    os.remove(path)
-            except FileNotFoundError:
-                pass
-            except Exception as e:
-                logging.warning("Problem removing existing files.  Transfer may not succeed: %s" % e)
-
         # We write files top-down.  If we're preserving permissions and we receive
         # a folder in some hierarchy that is not writable, we won't be able to create
         # anything inside it.
         self.folder_permission_change_list = []
 
-    def receive_data(self, s):
-        save_path = prefs.get_save_path()
+    def clean_existing_files(self):
+        logging.debug("Removing any existing files matching the pending transfer")
+        for name in self.op.top_dir_basenames:
+            path = Path(os.path.join(self.save_path, name))
+            self.rm_any(path)
 
-        path = os.path.join(save_path, s.relative_path)
+    def clean_current_top_dir_file(self):
+        if self.current_path is not None:
+            current = Path(self.current_path)
+            save = Path(self.save_path)
+
+            try:
+                relative = current.relative_to(save)
+                util.test_resolved_path_safety(relative.as_posix())
+            except (ValueError, ReceiveError) as e:
+                logging.critical("Partial file or directory from aborted transfer is invalid: %s" % str(e))
+                return
+
+            abs_top_dir = save.joinpath(relative.parts[0])
+            logging.debug("Removing partial file or directory: %s" % abs_top_dir)
+
+            self.rm_any(abs_top_dir)
+
+    def rm_any(self, path):
+        try:
+            try:
+                os.remove(path)
+            except IsADirectoryError:
+                shutil.rmtree(path)
+        except FileNotFoundError:
+            pass
+        except Exception as e:
+            logging.warning("Problem removing existing files: %s" % e)
+
+    def receive_data(self, s):
+        path = os.path.join(self.save_path, s.relative_path)
         if path != self.current_path:
             self.close_current_file()
             self.current_path = path
@@ -212,17 +229,13 @@ class FileReceiver(GObject.Object):
             self.current_type = s.file_type
             self.current_mtime = s.time.mtime
             self.current_mtime_usec = s.time.mtime_usec
-        if self.remaining_files == 0:
-            raise Exception(_("File count exceeds original request size"))
+            if not s.relative_path.startswith(tuple(self.op.top_dir_basenames)):
+                raise ReceiveError("File path is not descended from a valid toplevel directory: %s" % s.relative_path)
+            if self.op.remaining_count == 0:
+                raise ReceiveError("File count exceeds original request size")
 
         if not self.current_gfile:
-            # Check for valid path (pathlib.Path resolves both relative and symbolically-linked paths)
-            test_path = Path(path).resolve()
-            try:
-                test_path.relative_to(self.save_path_obj)
-            except ValueError:
-                raise ReceiveError(_("Resolved path is not valid: %s -> %s") % (path, str(test_path)), fatal=True)
-
+            util.test_resolved_path_safety(s.relative_path)
             self.current_gfile = Gio.File.new_for_path(path)
 
         if s.file_type == FileType.DIRECTORY:
@@ -230,7 +243,7 @@ class FileReceiver(GObject.Object):
         elif s.file_type == FileType.SYMBOLIC_LINK:
             make_symbolic_link(self.op, path, s.symlink_target)
         else:
-            if self.current_stream == None:
+            if self.current_stream is None:
                 self.current_stream = self.current_gfile.create(Gio.FileCreateFlags.NONE, None)
 
             if not s.chunk:
@@ -240,7 +253,7 @@ class FileReceiver(GObject.Object):
             self.op.progress_tracker.update_progress(len(s.chunk))
 
     def close_current_file(self):
-        if self.current_gfile == None:
+        if self.current_gfile is None:
             # First block received we self.close_current_file() with an empty path.
             return
 
@@ -280,7 +293,7 @@ class FileReceiver(GObject.Object):
         self.current_mode = 0
         self.current_path = None
         self.current_gfile = None
-        self.remaining_files -= 1
+        self.op.remaining_count -= 1
 
     def apply_folder_permissions(self):
         if self.preserve_perms:
@@ -379,7 +392,7 @@ def gather_file_info(op):
                 info = file.query_info(infos, Gio.FileQueryInfoFlags.NOFOLLOW_SYMLINKS, None)
                 basename = file.get_basename()
                 if len(uri_list) == 1:
-                    op.mime_if_single = info.get_content_type()
+                    op.mime_if_single = info.get_attribute_string(Gio.FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE)
 
                 if info and info.get_file_type() == FileType.DIRECTORY:
                     top_dir = file.get_parent().get_uri()
@@ -409,7 +422,7 @@ class OpProgressTracker():
         self.transfer_start_time = GLib.get_monotonic_time()
         self.last_update_time = self.transfer_start_time
 
-    @util._idle
+    @misc._idle
     def update_progress(self, size_read):
         self.total_transferred += size_read
 
